<?php

namespace App\Controller\CRUD;

use App\Util\WordpressUtil;
use Sonata\AdminBundle\Datagrid\ProxyQueryInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;

class PrestataireCRUDController extends CRUDController
{
    protected $wp;

    public function __construct(WordpressUtil $wp)
    {
        $this->wp = $wp;
    }

    /**
     * @param array|int|null  If $id is null, sync all the Prestataires
     */
    protected function syncToWordpress($id)
    {
        if ($this->wp->runSync($id)) {
            $this->addFlash('sonata_flash_success', 'Synchronisation vers Wordpress effectuée');
        }

        return new RedirectResponse(
            $this->admin->generateUrl('list', ['filter' => $this->admin->getFilterParameters()])
        );
    }

    public function syncAllWpAction()
    {
        return $this->syncToWordpress(null);
    }

    public function syncWpAction($id)
    {
        if (!$this->admin->hasSubject()) {
            throw new NotFoundHttpException(sprintf('Unable to find the Prestataire with id: %s', $id));
        }

        return $this->syncToWordpress($id);
    }

    public function batchActionSyncWp(ProxyQueryInterface $query)
    {
        $ids = [];
        $selected = $query->execute();

        foreach ($selected as $e) {
            $ids[] = $e->getId();
        }

        return $this->syncToWordpress($ids);
    }
}
